searchNodes=[{"doc":"Advent of Code solution module macro and helpers. This module contains the aoc/3 macro, which should be used to write a solution module for a given advent of code challenge. The intended use is to write your solution for day &lt;day&gt; , year &lt;year&gt; as follows: import AOC aoc &lt; year &gt; , &lt; day &gt; do def p1 ( input ) do # Part 1 solution goes here end def p2 ( input ) do # Part 1 solution goes here end end Defining a module with the aoc/3 macro has a few advantages: The AOC.IEx functions can be used to call your solutions in the module, making your life easier. Helper functions to access the input and examples (if present) are inserted into the generated module. Each of these advantages corresponds with a way to write your solution module, which we discuss below. Overall, the aoc/3 macro is intended to allow you to forego writing boilerplate code which is shared between all the solutions. Note that the code skeleton shown above can be generated by running mix aoc.gen or mix aoc . aoc/3 and use AOC Internally, aoc/3 generates a module with a predefined name ( Y&lt;year&gt;.D&lt;day&gt; ) which contains a use AOC, day: &lt;day&gt;, year: &lt;year&gt; statement. In turn, the __using__/1 macro defined in this module is responsible for generating helper functions. Thus, if you prefer to use a different naming scheme than the one imposed by aoc/3 , you can write your module as follows: defmodule MySolution do use AOC , day : &lt; day &gt; , year : &lt; year &gt; ... end When the AOC module is used like this, the helper functions defined below are still usable, but the helpers defined in AOC.IEx will not work. IEx The AOC.IEx module defines various helpers which facilitate testing the code in this module within iex, the elixir shell. For instance, if you write your solution for part 1 as follows: aoc &lt; year &gt; , &lt; day &gt; do def p1 ( input ) do # solutions go here end end You can use AOC.IEx.p1e/1 to call p1 with the example input of that day and AOC.IEx.p1i/1 to call p1 with the puzzle input of that day. Similar functions are available for p2 . Helper functions You can also directly access the puzzle input or examples for a day inside an aoc/3 module. This module defines various functions such as input_path/2 , input_string/2 , input_stream/2 and their example_*/2 counter parts. Helpers are inserted inside the generated module which call these functions with the module's day / year. Thus, if you call input_path() inside your solution module, it will call input_path/2 for you with the module's day and year, obtaining the path to the appropriate input file. The following table provides an overview of the inserted functions and their counterparts in this module: Generated Calls input_path/0 input_path/2 input_string/0 input_string/2 input_stream/0 input_stream/2 example_path/0 example_path/2 example_string/0 example_string/2 example_stream/0 example_stream/2 The generated functions are overridable, i.e. you can define your own version of these functions which will overwrite the generated function. This is handy to do something like the following: def input_stream , do : super ( ) |&gt; Stream . map ( &amp; String . to_integer / 1 )","ref":"AOC.html","title":"AOC","type":"module"},{"doc":"Generate an advent of code solution module for a given year and day. The generated module will be named Y&lt;year&gt;.D&lt;day&gt; . use AOC will be injected in the body of the module, so that the input helpers described in the module documentation are available. Examples import AOC aoc 2020 , 1 do def some_function do :foo end end is equivalent to: defmodule Y2020.D1 do use AOC , year : 2020 , day : 1 def some_function do :foo end end","ref":"AOC.html#aoc/3","title":"AOC.aoc/3","type":"macro"},{"doc":"Get the example path for year , day . Obtains the path where mix aoc.get stores the input for year , day . This path defaults to input/&lt;year&gt;_&lt;day&gt;_example.txt , but can be customized. Please refer to the mix aoc.get documentation for more information.","ref":"AOC.html#example_path/2","title":"AOC.example_path/2","type":"function"},{"doc":"Stream the contents of the example for year , day . The stream is created by calling File.stream!/1 on the path returned by example_path/2 . Afterwards, String.trim/1 is mapped over the stream (using Stream.map/2 ), to remove trailing newlines and whitespace.","ref":"AOC.html#example_stream/2","title":"AOC.example_stream/2","type":"function"},{"doc":"Get the example contents of year , day . Obtained by calling File.read!/1 on the path returned by example_path/2 . String.trim_trailing/1 is called on the resulting string to remove trailing whitespace.","ref":"AOC.html#example_string/2","title":"AOC.example_string/2","type":"function"},{"doc":"Get the input path for year , day . Obtains the path where mix aoc.get stores the input for year , day . This path defaults to input/&lt;year&gt;_&lt;day&gt;.txt , but can be customized. Please refer to the mix aoc.get documentation for more information.","ref":"AOC.html#input_path/2","title":"AOC.input_path/2","type":"function"},{"doc":"Stream the contents of the input for year , day . The stream is created by calling File.stream!/1 on the path returned by input_path/2 . Afterwards, String.trim/1 is mapped over the stream (using Stream.map/2 ), to remove trailing newlines and whitespace.","ref":"AOC.html#input_stream/2","title":"AOC.input_stream/2","type":"function"},{"doc":"Get the input contents of year , day . Obtained by calling File.read!/1 on the path returned by input_path/2 . String.trim_trailing/1 is called on the resulting string to remove trailing whitespace.","ref":"AOC.html#input_string/2","title":"AOC.input_string/2","type":"function"},{"doc":"IEx helpers for advent of code. This module contains various helpers that make it easy to call procedures in your solution modules. It is intended to be used while testing solutions in iex, the elixir shell. In order to avoid prefixing all calls with AOC.IEx , we recommend adding import AOC.IEx to your .iex.exs file . Requirements and AOC.aoc/3 In order to find a module for a given day and year, this module expects the module to have the name Y&lt;year&gt;.D&lt;day&gt; . This is always the case if the AOC.aoc/3 macro was used to build the solution module (and thus the case if the template generated by mix aoc.gen or mix aoc is used). Furthermore, it is expected that the solutions for part 1 and part 2 are defined in non-private functions named p1 and p2 . These functions must either accept one argument (the puzzle input, provided as a string) or no inputs at all. Functions in this module This module provides the p1/2 and p2/2 functions to call the p1 and p2 functions of your solution module from within iex. It is assumed that the p1 and p2 functions either accept one argument or no arguments at all: If p1 or p2 accept the puzzle input encoded as a string as an argument, p1e/1 , p2e/1 , p1i/1 and p2i/1 can be used to call these functions with the example ( p1e/1 , p2e/1 ) or puzzle input ( p1i/1 , p2i/1 ). These functions call p1/2 or p2/2 with the result of example_string/1 and input_string/1 internally. p1/2 and p2/2 can also be used directly to call p1 or p2 with alternative input (e.g. with additional example input). If p1 and p2 accept no arguments, p1/2 and p2/2 can still be used. They will ignore the provided input and call p1 or p2 directly. mod/1 can be used to obtain the current solution module, which is useful if you wish to test other functions in your solution module. Moreover, example_path/1 , input_path/1 , example_string/1 , input_string/1 , example_stream/1 and input_stream/1 can be used to experiment with the puzzle input and example input retrieved by mix aoc.get or mix aoc inside iex. Specifying the puzzle date The functions in this module all select a puzzle (or more specifically, its solution module, input or example) based on the current time. For instance, the p1/2 function calls the p1 function of the solution module that corresponds to the current day. The current day (and year) is determined by NaiveDateTime.local_now/0 . If it is past midnight, or if you wish to solve an older challenge, there are a few options at your disposal: Each function in this module accepts an optional keyword list through which the year and day can be specified. For instance, if you wish to run part 1 of of 8 december 1991, you could write the following code: p1(&lt;input&gt;, year: 1991, day: 8) . If you omit the day or year, the current day or year is used by default. p1(&lt;input&gt;, day: 8) would, for instance, call part 1 of day 8 of the current year. The year and day can be configured through the :advent_of_code_utils application environment. For instance, you can set the year to 1991 and the day to 8 by placing the following in your config/config.exs : import Config config :advent_of_code_utils , day : 8 , year : 1991 Both of these options can be combined. You can, for instance, set the year in config/config.exs and select the day when calling p1/2 or any other function in this module. To summarise, the day or year is determined according to the following rules: If year or day is passed as part of the keyword list argument, it is always used. If :year or :day is present in the :advent_of_code_utils application environment, it is used. The year or day returned by NaiveDateTime.local_now/0 is used. Automatic recompilation It is often necessary to recompile the current mix project before running code. To avoid repeatedly calling IEx.Helpers.recompile/1 , the various p* functions in this module and mod/1 will automatically recompile the current mix project (with IEx.Helpers.recompile/1 ) when :auto_compile? is set to true in the :advent_of_code_utils application environment: import Config config :advent_of_code_utils , auto_compile? : true","ref":"AOC.IEx.html","title":"AOC.IEx","type":"module"},{"doc":"Obtain the path of the example input of the current puzzle. Calls AOC.example_path/2 with &lt;day&gt; and &lt;year&gt;. If not present in the options list, day and year are fetched from the application environment or based on the local time. Refer to the module documentation for additional information.","ref":"AOC.IEx.html#example_path/1","title":"AOC.IEx.example_path/1","type":"function"},{"doc":"Obtain the example input of the current puzzle as a stream. Calls AOC.example_stream/2 with &lt;day&gt; and &lt;year&gt;. If not present in the options list, day and year are fetched from the application environment or based on the local time. Refer to the module documentation for additional information.","ref":"AOC.IEx.html#example_stream/1","title":"AOC.IEx.example_stream/1","type":"function"},{"doc":"Obtain the example input of the current puzzle as a string. Calls AOC.example_string/2 with &lt;day&gt; and &lt;year&gt;. If not present in the options list, day and year are fetched from the application environment or based on the local time. Refer to the module documentation for additional information.","ref":"AOC.IEx.html#example_string/1","title":"AOC.IEx.example_string/1","type":"function"},{"doc":"Obtain the path of the input for the current puzzle. Calls AOC.input_path/2 with &lt;day&gt; and &lt;year&gt;. If not present in the options list, day and year are fetched from the application environment or based on the local time. Refer to the module documentation for additional information.","ref":"AOC.IEx.html#input_path/1","title":"AOC.IEx.input_path/1","type":"function"},{"doc":"Obtain the puzzle input of the current puzzle as a stream. Calls AOC.input_stream/2 with &lt;day&gt; and &lt;year&gt;. If not present in the options list, day and year are fetched from the application environment or based on the local time. Refer to the module documentation for additional information.","ref":"AOC.IEx.html#input_stream/1","title":"AOC.IEx.input_stream/1","type":"function"},{"doc":"Obtain the puzzle input of the current puzzle as a string. Calls AOC.input_string/2 with &lt;day&gt; and &lt;year&gt;. If not present in the options list, day and year are fetched from the application environment or based on the local time. Refer to the module documentation for additional information.","ref":"AOC.IEx.html#input_string/1","title":"AOC.IEx.input_string/1","type":"function"},{"doc":"Get the module name for the currently configured puzzle. If not present in the options list, day and year are fetched from the application environment or based on the local time. Refer to the module documentation for additional information. This function may cause recompilation if auto_compile? is enabled. Examples iex&gt; mod ( year : 1991 , day : 8 ) Y1991.D8 iex&gt; Application . put_env ( :advent_of_code_utils , :year , 1991 ) iex&gt; Application . put_env ( :advent_of_code_utils , :day , 8 ) iex&gt; mod ( ) Y1991.D8 iex&gt; mod ( day : 9 ) Y1991.D9 iex&gt; mod ( year : 2000 ) Y2000.D8 iex&gt; Application . put_env ( :advent_of_code_utils , :year , 2000 ) iex&gt; Application . put_env ( :advent_of_code_utils , :day , 3 ) iex&gt; mod ( ) Y2000.D3","ref":"AOC.IEx.html#mod/1","title":"AOC.IEx.mod/1","type":"function"},{"doc":"Call part 1 of the current puzzle with the given input. If available, Y&lt;year&gt;.D&lt;day&gt;.p1/1 is called with input . Otherwise, the input is ignored and Y&lt;year&gt;.D&lt;day&gt;.p1() is called. If neither is available, an error is raised. The input may be omitted, in which case it defaults to nil . This is useful when p1 does not accept any arguments. If you wish to omit the input but provide a year or day, you need to manually pass nil as input. If not present in the options list, day and year are fetched from the application environment or based on the local time. Refer to the module documentation for additional information. This function may cause recompilation if auto_compile? is enabled.","ref":"AOC.IEx.html#p1/2","title":"AOC.IEx.p1/2","type":"function"},{"doc":"Call part 1 of the current puzzle with its example input. Uses p1/3 and example_path/1 to call Y&lt;year&gt;.D&lt;day&gt;.p1/1 with the example input of &lt;year&gt; and &lt;day&gt;. If not present in the options list, day and year are fetched from the application environment or based on the local time. Refer to the module documentation for additional information. This function may cause recompilation if auto_compile? is enabled.","ref":"AOC.IEx.html#p1e/1","title":"AOC.IEx.p1e/1","type":"function"},{"doc":"Call part 1 of the current puzzle with its input. Uses p1/3 and input_path/1 to call Y&lt;year&gt;.D&lt;day&gt;.p1/1 with the input of &lt;year&gt; and &lt;day&gt;. If not present in the options list, day and year are fetched from the application environment or based on the local time. Refer to the module documentation for additional information. This function may cause recompilation if auto_compile? is enabled.","ref":"AOC.IEx.html#p1i/1","title":"AOC.IEx.p1i/1","type":"function"},{"doc":"Call part 2 of the current puzzle with the given input. If available, Y&lt;year&gt;.D&lt;day&gt;.p2/1 is called with input . Otherwise, the input is ignored and Y&lt;year&gt;.D&lt;day&gt;.p2() is called. If neither is available, an error is raised. The input may be omitted, in which case it defaults to nil . This is useful when p1 does not accept any arguments. If you wish to omit the input but provide a year or day, you need to manually pass nil as input. If not present in the options list, day and year are fetched from the application environment or based on the local time. Refer to the module documentation for additional information. This function may cause recompilation if auto_compile? is enabled.","ref":"AOC.IEx.html#p2/2","title":"AOC.IEx.p2/2","type":"function"},{"doc":"Call part 2 of the current puzzle with its example input. Uses p2/3 and example_path/1 to call Y&lt;year&gt;.D&lt;day&gt;.p2/1 with the example input of &lt;year&gt; and &lt;day&gt;. If not present in the options list, day and year are fetched from the application environment or based on the local time. Refer to the module documentation for additional information. This function may cause recompilation if auto_compile? is enabled.","ref":"AOC.IEx.html#p2e/1","title":"AOC.IEx.p2e/1","type":"function"},{"doc":"Call part 2 of the current puzzle with its input. Uses p2/3 and input_path/1 to call Y&lt;year&gt;.D&lt;day&gt;.p2/1 with the input of &lt;year&gt; and &lt;day&gt;. If not present in the options list, day and year are fetched from the application environment or based on the local time. Refer to the module documentation for additional information. This function may cause recompilation if auto_compile? is enabled.","ref":"AOC.IEx.html#p2i/1","title":"AOC.IEx.p2i/1","type":"function"},{"doc":"Create a code skeleton and fetch input for the AOC challenge of a day / year. This mix task runs mix aoc.gen followed by mix aoc.get . Afterwards, it prints the url of today's challenge. mix aoc.gen This task generates a code skeleton for the given day and year in lib/&lt;year&gt;/&lt;day&gt;.ex , which uses the AOC.aoc/3 macro. To customise the path where code is stored, refer to the mix aoc.gen documentation. mix aoc.get This task fetches the input and example input for a given day and year and stores it in input/&lt;year&gt;_&lt;day&gt;.txt and input/&lt;year&gt;_&lt;day&gt;_example.txt . In order for this task to work, your session cookie should be passed as a command line argument or set up in the :advent_of_code_utils application environment. Please refer to the mix aoc.get documentation for more information. Command-line arguments -s or --session : Specify the session cookie. -y or --year : Specify the year. -d or --day : Specify the day. --example : Fetch example input (the default) --no-example : Do not fetch example input All of these options take precedence over their application environment counterparts.","ref":"Mix.Tasks.Aoc.html","title":"Mix.Tasks.Aoc","type":"task"},{"doc":"Callback implementation for Mix.Task.run/1 .","ref":"Mix.Tasks.Aoc.html#run/1","title":"Mix.Tasks.Aoc.run/1","type":"function"},{"doc":"Create a code skeleton for the AOC challenge of a given day / year. This mix task creates a code skeleton for the advent of code challenge of a specific day. The day and year of the challenge can be passed as command-line arguments or be set in the advent_of_code_utils application configuration. When neither of these are present, the current date is used. By default, this task stores the generated code skeleton in lib/&lt;year&gt;/&lt;day&gt;.ex . If this file already exists, an error is returned and no files are created. The destination path can be modified by setting the value of :code_path in the advent_of_code_utils application configuration. This value should be set to a string which may contain :year and :day . These values will be replaced by the day and year for which a skeleton is generated. For instance, the following configuration will store the generated code in lib/aoc_&lt;year&gt;_day_&lt;day&gt;.ex : config :advent_of_code_utils , :code_path , &quot;lib/aoc_:year_day_:day.ex&quot; Command-line arguments The options below take precedence over values defined in the application configuration. -y or --year : Specify the year. -d or --day : Specify the day.","ref":"Mix.Tasks.Aoc.Gen.html","title":"Mix.Tasks.Aoc.Gen","type":"task"},{"doc":"Callback implementation for Mix.Task.run/1 .","ref":"Mix.Tasks.Aoc.Gen.html#run/1","title":"Mix.Tasks.Aoc.Gen.run/1","type":"function"},{"doc":"Fetch the input and example input of the AOC challenge for a given day / year. This mix task fetches the input and example input for the advent of code challenge of a specific day. The day and year of the challenge can be passed as command-line arguments or be set in the advent_of_code_utils application configuration. When neither is present, the current date is used. By default, this task stores the fetched input data in input/&lt;year&gt;_&lt;day&gt;.txt . The fetched example is stored in input/&lt;year&gt;_&lt;day&gt;_example.txt . If a file already exists, the matching input is not fetched. The destination paths can be modified by setting the value of :input_path or :example_path in the advent_of_code_utils application configuration. These values should be set to a string which may contain :year and :day . These values will be replaced by the day and year of which the input is fetched. For instance, the following configuration will store the fetched input data in my_input/&lt;year&gt;/&lt;day&gt;.input : config :advent_of_code_utils , :code_path , &quot;my_input/:year/:day.input&quot; Session cookie In order to fetch your input, this task needs your advent of code session cookie. This can be obtained by investigating your cookies after logging in on the advent of code website. The cookie can be stored inside your config/config.exs file (e.g. config, :advent_of_code_utils, :session, &quot;&lt;your cookie here&gt;&quot; ) or it can be passed as a command-line argument. If no cookie is present, the input can not be fetched. Example input The example input of a given day is fetched by parsing the challenge webpage of a given day and returning the first code example found on that page. This is generally the example input of that day. As this method is not foolproof, it is sometimes necessary to modify the example file by hand. If you do not wish to fetch example input, you can pass the --no-example flag to this task, or you can set fetch_example to false in the advent_of_code_utils application configuration. Command-line arguments The options below take precedence over values defined in the application configuration. -s or --session : Specify the session cookie. -y or --year : Specify the year. -d or --day : Specify the day. --no-example : Do not fetch example input --example : Fetch example input","ref":"Mix.Tasks.Aoc.Get.html","title":"Mix.Tasks.Aoc.Get","type":"task"},{"doc":"","ref":"Mix.Tasks.Aoc.Get.html#find_example/1","title":"Mix.Tasks.Aoc.Get.find_example/1","type":"function"},{"doc":"Callback implementation for Mix.Task.run/1 .","ref":"Mix.Tasks.Aoc.Get.html#run/1","title":"Mix.Tasks.Aoc.Get.run/1","type":"function"},{"doc":"Input fetching and boilerplate generation for Advent of Code . The goal of this project is to eliminate most of the manual labor involved with working on the yearly Advent of Code challenges. As a sample, this is the workflow you'd use when working on the challenge of the first of December 2020: $ mix aoc * Creating: lib/2020/1.ex * Creating: input/2020_1.txt * Creating: input/2020_1_example.txt Today&#39;s challenge can be found at: https://adventofcode.com/2020/day/1 Afterwards, lib/2020/1.ex will look as follows: import AOC aoc 2020 , 1 do def p1 ( input ) do end def p2 ( input ) do end end While solving your challenge, you can use the AOC.IEx.p1e/1 and AOC.IEx.p2e/1 helpers in iex to test your solution so far with the example input. Once ready, you can use AOC.IEx.p1i/1 and AOC.IEx.p2i/1 to run your solution on your puzzle input. These helpers can also be set up to automatically recompile your mix project. All of this is configurable so that you can adjust this project to fit your own workflow. For instance, you can forego the iex helpers, and instead access the contents of the fetched input by using input_path/0 , input_string/0 or input_stream/0 . Example input is available through example_path/0 , example_string/0 or example_stream/0 . Check out the docs for more information!","ref":"readme.html","title":"Advent of Code Utils","type":"extras"},{"doc":"Add advent_of_code_utils to your list of dependencies in mix.exs : def deps do [ { :advent_of_code_utils , &quot;~&gt; 3.0&quot; } ] end Configure your Advent of Code project in config/config.exs : Store your session cookie in config/config.exs . You can find this by inspecting your cookies after logging in to the advent of code website. config :advent_of_code_utils , session : &quot;&lt;your cookie&gt;&quot; (Optional) Set auto_compile? to true if you want the various AOC.IEx.p* helpers to recompile your project: config :advent_of_code_utils , auto_compile? : true (Optional) Configure iex to display charlists as lists. This will prevent lists like [99, 97, 116] to show up as 'cat' : config :iex , inspect : [ charlists : :as_lists ] If you follow these steps, your config/config.exs should look as follows: import Config config :advent_of_code_utils , auto_compile? : true , session : &quot;&lt;your session cookie&gt;&quot; config :iex , inspect : [ charlists : :as_lists ] (Optional) Add import AOC.IEx to your .iex.exs file . This allows you to use the utilities defined in AOC.IEx without specifying the module name. (Optional) Add input/ to your .gitignore file if you use git. Now that you are set up, you can use mix aoc to work on today's challenge. The day and year of a challenge can be passed in various ways, so this project can still be used when working on older challenges. If you only want to use this application to fetch the input of challenge, without generating any code, you can use mix aoc.get instead of mix aoc . Additionally, you can skip most of the optional steps above.","ref":"readme.html#setup-use","title":"Advent of Code Utils - Setup &amp; Use","type":"extras"},{"doc":"Besides fetching input, mix aoc.get and mix aoc will also fetch example input for the given day. This is done by reading the first code example on the challenge webpage, which is generally that day's example input. Since this method is not 100% reliable, you may you wish to disable this behaviour. This can be done by passing the --no-example flag to mix aoc or mix aoc.get or by setting fetch_example to false in your config.exs file.","ref":"readme.html#example-input","title":"Advent of Code Utils - Example Input","type":"extras"},{"doc":"This project grew from a collection of utilities I wrote for myself when working on advent of code. I polished these utilities, but it is possible some bugs are still present. If you run into any issue, feel free to create an issue on GitHub .","ref":"readme.html#issues","title":"Advent of Code Utils - Issues","type":"extras"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"AOC.IEx has been reworked. All functions accept an opts keyword list used to specify year or day when needed. p1 and p2 now accept puzzle input as an argument, making it easier to switch between different inputs or examples. Introduce p1e , p1i , p2e and p2i to facilitate calling p1 or p2 with example or puzzle input. Introduce wrapper functions example_path/1 , input_path/1 , example_string/1 , input_string/1 , example_stream/1 and input_stream/1 to facilitate experimentation inside iex. Update AOC documentation to reflect to new workflow. Update mix aoc.gen to generate a template more suited to new workflow.","ref":"changelog.html#3-0-0","title":"Changelog - 3.0.0","type":"extras"},{"doc":"Pass user agent when using mix aoc.get","ref":"changelog.html#2-0-2","title":"Changelog - 2.0.2","type":"extras"},{"doc":"Update ex_doc, add typepsecs to public functions.","ref":"changelog.html#2-0-1","title":"Changelog - 2.0.1","type":"extras"},{"doc":"input_string and example_string now both call String.trim_trailing/1 on the returned string.","ref":"changelog.html#2-0-0","title":"Changelog - 2.0.0","type":"extras"},{"doc":"Download example input when using mix aoc.get . Add example_* functions to AOC module.","ref":"changelog.html#1-1-0","title":"Changelog - 1.1.0","type":"extras"},{"doc":"Make input_* functions public ( def instead of defp ) to facilitate debugging.","ref":"changelog.html#1-0-1","title":"Changelog - 1.0.1","type":"extras"},{"doc":"Initial stable release","ref":"changelog.html#1-0-0","title":"Changelog - 1.0.0","type":"extras"}]